@@ -35,11 +35,11 @@
 #define TYPE_FLEXI 3		/* Local Flexi X server */
 #define TYPE_FLEXI_XNEST 4	/* Local Flexi Xnest server */
 
-#define SERVER_IS_LOCAL(d) (d->type == TYPE_LOCAL || \
-			    d->type == TYPE_FLEXI || \
-			    d->type == TYPE_FLEXI_XNEST)
-#define SERVER_IS_FLEXI(d) (d->type == TYPE_FLEXI || \
-			    d->type == TYPE_FLEXI_XNEST)
+#define SERVER_IS_LOCAL(d) ((d)->type == TYPE_LOCAL || \
+			    (d)->type == TYPE_FLEXI || \
+			    (d)->type == TYPE_FLEXI_XNEST)
+#define SERVER_IS_FLEXI(d) ((d)->type == TYPE_FLEXI || \
+			    (d)->type == TYPE_FLEXI_XNEST)
 
 /* These are the servstat values, also used as server
  * process exit codes */
@@ -61,6 +61,7 @@
 #define DISPLAY_CHOSEN 20	/* successful chooser session,
 				   restart display */
 #define DISPLAY_XFAILED 64	/* X failed */
+#define DISPLAY_RESTARTGREETER 127 /* Restart greeter */
 #define DISPLAY_RESTARTGDM 128	/* Restart GDM */
 
 enum {
@@ -76,7 +77,7 @@ enum {
 
 /* This will change if there are incompatible
  * protocol changes */
-#define GDM_GREETER_PROTOCOL_VERSION "1"
+#define GDM_GREETER_PROTOCOL_VERSION "2"
 
 #define GDM_MSG        'D'
 #define GDM_NOECHO     'U'
@@ -104,6 +105,9 @@ enum {
 #define GDM_READPIC    '%' /* Send a user picture in a temp file */
 #define GDM_ERRBOX     'e' /* Puts string in the error box */
 #define GDM_ERRDLG     'E' /* Puts string up in an error dialog */
+#define GDM_NOFOCUS    'f' /* Don't focus the login window (optional) */
+#define GDM_FOCUS      'F' /* Allow focus on the login window again (optional) */
+#define GDM_SAVEDIE    '!' /* Save wm order and die (and set busy cursor) */
 
 /* Different login interruptions */
 #define GDM_INTERRUPT_TIMED_LOGIN 'T'
@@ -114,14 +118,15 @@ enum {
 #define PIPE_SIZE 4096
 
 /* Configuration constants */
-#define GDM_KEY_CHOOSER "daemon/Chooser=" EXPANDED_BINDIR "/gdmchooser --disable-sound --disable-crash-dialog"
+#define GDM_KEY_CHOOSER "daemon/Chooser=" EXPANDED_BINDIR "/gdmchooser"
 /* This defaults to true for backward compatibility,
  * it will not actually do automatic login since the AutomaticLogin defaults
  * to nothing */
 #define GDM_KEY_AUTOMATICLOGIN_ENABLE "daemon/AutomaticLoginEnable=true"
 #define GDM_KEY_AUTOMATICLOGIN "daemon/AutomaticLogin="
 #define GDM_KEY_ALWAYSRESTARTSERVER "daemon/AlwaysRestartServer=false"
-#define GDM_KEY_GREETER "daemon/Greeter=" EXPANDED_BINDIR "/gdmlogin --disable-sound --disable-crash-dialog"
+#define GDM_KEY_GREETER "daemon/Greeter=" EXPANDED_BINDIR "/gdmlogin"
+#define GDM_KEY_REMOTEGREETER "daemon/RemoteGreeter=" EXPANDED_BINDIR "/gdmlogin"
 #define GDM_KEY_GROUP "daemon/Group=gdm"
 #define GDM_KEY_HALT "daemon/HaltCommand=/sbin/shutdown -h now"
 #define GDM_KEY_INITDIR "daemon/DisplayInitDir=" EXPANDED_SYSCONFDIR "/gdm/Init"
@@ -154,7 +159,7 @@ enum {
 
 #define GDM_KEY_STANDARD_XSERVER "daemon/StandardXServer=/usr/bin/X11/X"
 #define GDM_KEY_FLEXIBLE_XSERVERS "daemon/FlexibleXServers=5"
-#define GDM_KEY_XNEST "daemon/Xnest=/usr/bin/X11/Xnest -name Xnest"
+#define GDM_KEY_XNEST "daemon/Xnest=/usr/bin/X11/Xnest -name Xnest -kb"
 
 /* Per server definitions */
 #define GDM_KEY_SERVER_NAME "name=Standard server"
@@ -162,7 +167,9 @@ enum {
 /* runnable as flexi server */
 #define GDM_KEY_SERVER_FLEXIBLE "flexible=true"
 /* choosable from the login screen */
-#define GDM_KEY_SERVER_CHOOSABLE "choosable=true"
+#define GDM_KEY_SERVER_CHOOSABLE "choosable=false"
+/* Login is handled by gdm, otherwise it's a remote server */
+#define GDM_KEY_SERVER_HANDLED "handled=true"
 
 #define GDM_KEY_ALLOWROOT "security/AllowRoot=true"
 #define GDM_KEY_ALLOWREMOTEROOT "security/AllowRemoteRoot=true"
@@ -190,6 +197,7 @@ enum {
 
 #define GDM_KEY_BROWSER "greeter/Browser=false"
 #define GDM_KEY_EXCLUDE "greeter/Exclude=bin,daemon,adm,lp,sync,shutdown,halt,mail,news,uucp,operator,nobody,gdm,postgres,pvm"
+#define GDM_KEY_MINIMALUID "greeter/MinimalUID=100"
 #define GDM_KEY_FACE "greeter/DefaultFace=" EXPANDED_PIXMAPDIR "nobody.png"
 #define GDM_KEY_FACEDIR "greeter/GlobalFaceDir=" EXPANDED_DATADIR "/faces/"
 #define GDM_KEY_ICON "greeter/Icon=" EXPANDED_PIXMAPDIR "/gdm.xpm"
@@ -198,16 +206,15 @@ enum {
 #define GDM_KEY_LOGO "greeter/Logo=" EXPANDED_PIXMAPDIR "/gnome-logo-large.png"
 #define GDM_KEY_QUIVER "greeter/Quiver=true"
 #define GDM_KEY_SYSMENU "greeter/SystemMenu=true"
-#define GDM_KEY_CONFIGURATOR "daemon/Configurator=" EXPANDED_GDMCONFIGDIR "/gdmconfig --disable-sound --disable-crash-dialog"
+#define GDM_KEY_CONFIGURATOR "daemon/Configurator=" EXPANDED_GDMCONFIGDIR "/gdmsetup --disable-sound --disable-crash-dialog"
 #define GDM_KEY_CONFIG_AVAILABLE "greeter/ConfigAvailable=true"
 #define GDM_KEY_TITLE_BAR "greeter/TitleBar=true"
-/* translated string gets HATE defaults */
-#define GDM_KEY_WELCOME_TR "greeter/Welcome"
 #define GDM_KEY_WELCOME "greeter/Welcome=Welcome to %n"
 #define GDM_KEY_XINERAMASCREEN "greeter/XineramaScreen=0"
 #define GDM_KEY_BACKGROUNDPROG "greeter/BackgroundProgram="
+#define GDM_KEY_RUNBACKGROUNDPROGALWAYS "greeter/RunBackgroundProgramAlways=false"
 #define GDM_KEY_BACKGROUNDIMAGE "greeter/BackgroundImage="
-#define GDM_KEY_BACKGROUNDCOLOR "greeter/BackgroundColor=#007777"
+#define GDM_KEY_BACKGROUNDCOLOR "greeter/BackgroundColor=#363047"
 #define GDM_KEY_BACKGROUNDTYPE "greeter/BackgroundType=2"
 #define GDM_KEY_BACKGROUNDSCALETOFIT "greeter/BackgroundScaleToFit=true"
 #define GDM_KEY_BACKGROUNDREMOTEONLYCOLOR "greeter/BackgroundRemoteOnlyColor=true"
@@ -216,6 +223,9 @@ enum {
 #define GDM_KEY_POSITIONX "greeter/PositionX=0"
 #define GDM_KEY_POSITIONY "greeter/PositionY=0"
 #define GDM_KEY_USE_24_CLOCK "greeter/Use24Clock=false"
+#define GDM_KEY_ENTRY_CIRCLES "greeter/UseCirclesInEntry=false"
+#define GDM_KEY_GRAPHICAL_THEME "greeter/GraphicalTheme=circles"
+#define GDM_KEY_GRAPHICAL_THEME_DIR "greeter/GraphicalThemeDir=" EXPANDED_DATADIR "/gdm/themes/"
 
 #define GDM_KEY_SCAN "chooser/ScanTime=3"
 #define GDM_KEY_HOST "chooser/DefaultHostImg=" EXPANDED_PIXMAPDIR "/nohost.png"
@@ -279,6 +289,8 @@ struct _GdmDisplay {
     pid_t chooserpid;
     time_t acctime;
 
+    gboolean handled;
+
 #ifdef __linux__
     int vt;
 #endif
@@ -306,12 +318,21 @@ struct _GdmDisplay {
     int screeny;
     int screenwidth; /* Note 0 means use the gdk size */
     int screenheight;
+    int lrh_offsetx; /* lower right hand corner x offset */
+    int lrh_offsety; /* lower right hand corner y offset */
 
     /* Flexi stuff */
     char *xnest_disp;
     char *xnest_auth_file;
     uid_t server_uid;
     GdmConnection *socket_conn;
+
+    /* Notification connection */
+    int master_notify_fd;  /* write part of the connection */
+    int slave_notify_fd; /* read part of the connection */
+
+    /* order in the Xservers file for sessreg, -1 if unset yet */
+    int x_servers_order;
 };
 
 typedef struct _GdmXServer GdmXServer;
@@ -321,6 +342,7 @@ struct _GdmXServer {
 	char *command;
 	gboolean flexible;
 	gboolean choosable; /* not implemented yet */
+	gboolean handled;
 };
 
 typedef struct _GdmIndirectDisplay GdmIndirectDisplay;
@@ -371,18 +393,6 @@ enum {
 	GDM_XDMCP_LAST_OPCODE /*just a marker, not an opcode */
 };
 
-typedef gboolean (*GDMSignalFunc) (gint8	signal,
-				 gpointer	data);
-guint	gdm_signal_add		(gint8		signal,
-				 GDMSignalFunc	function,
-				 gpointer    	data);
-guint   gdm_signal_add_full     (gint           priority,
-				 gint8          signal,
-				 GDMSignalFunc  function,
-				 gpointer       data,
-				 GDestroyNotify destroy);
-void    gdm_signal_notify       (gint8          signal);
-
 /* If id == NULL, then get the first X server */
 GdmXServer *	gdm_find_x_server	(const char *id);
 
@@ -411,6 +421,47 @@ GdmXServer *	gdm_find_x_server	(const char *id);
 #define GDM_SOP_FLEXI_OK     "FLEXI_OK" /* <slave pid> */
 #define GDM_SOP_SOFT_RESTART "SOFT_RESTART" /* no arguments */
 #define GDM_SOP_START_NEXT_LOCAL "START_NEXT_LOCAL" /* no arguments */
+#define GDM_SOP_HUP_ALL_GREETERS "HUP_ALL_GREETERS" /* no arguments */
+/* sometimes we can't do a syslog so we tell the main daemon */
+#define GDM_SOP_SYSLOG "SYSLOG" /* <pid> <type> <message> */
+/* write out a sessreg (xdm) compatible Xservers file
+ * in the ServAuthDir as <name>.Xservers */
+#define GDM_SOP_WRITE_X_SERVERS "WRITE_X_SERVERS" /* <slave pid> */
+/* All X servers should be restarted rather then regenerated.  Useful
+ * if you have updated the X configuration.  Note that this happens
+ * only when the user logs out or when we otherwise would have restarted
+ * a server, nothing is done by this command. */
+#define GDM_SOP_DIRTY_SERVERS "DIRTY_SERVERS"  /* no arguments */
+/* restart all servers that people aren't logged in on.  Maybe you may not
+ * want to do this on every change of X server config since this may cause
+ * flicker on screen and jumping around on the vt.  Perhaps useful to do
+ * by asking the user if they want to do that.  Note that this will not
+ * kill any logged in sessions. */
+#define GDM_SOP_SOFT_RESTART_SERVERS "SOFT_RESTART_SERVERS"  /* no arguments */
+
+/* Notification protocol */
+/* keys */
+#define GDM_NOTIFY_ALLOWREMOTEROOT "AllowRemoteRoot" /* <true/false as int> */
+#define GDM_NOTIFY_ALLOWROOT "AllowRoot" /* <true/false as int> */
+#define GDM_NOTIFY_ALLOWREMOTEAUTOLOGIN "AllowRemoteAutoLogin" /* <true/false as int> */
+#define GDM_NOTIFY_SYSMENU "SystemMenu" /* <true/false as int> */
+#define GDM_NOTIFY_CONFIG_AVAILABLE "ConfigAvailable" /* <true/false as int> */
+#define GDM_NOTIFY_RETRYDELAY "RetryDelay" /* <seconds> */
+#define GDM_NOTIFY_GREETER "Greeter" /* <greeter binary> */
+#define GDM_NOTIFY_REMOTEGREETER "RemoteGreeter" /* <greeter binary> */
+#define GDM_NOTIFY_TIMED_LOGIN "TimedLogin" /* <login> */
+#define GDM_NOTIFY_TIMED_LOGIN_DELAY "TimedLoginDelay" /* <seconds> */
+
+/* commands */
+#define GDM_NOTIFY_DIRTY_SERVERS "DIRTY_SERVERS"
+#define GDM_NOTIFY_SOFT_RESTART_SERVERS "SOFT_RESTART_SERVERS"
+
+/* Ack for a slave message */
+#define GDM_SLAVE_NOTIFY_ACK 'A'
+/* Update this key */
+#define GDM_SLAVE_NOTIFY_KEY '!'
+/* notify a command */
+#define GDM_SLAVE_NOTIFY_COMMAND '#'
 
 #define GDM_SUP_SOCKET "/tmp/.gdm_socket"
 
@@ -467,20 +518,21 @@ GdmXServer *	gdm_find_x_server	(const char *id);
  *      100 = Not authenticated
  *      999 = Unknown error
  */
-#define GDM_SUP_FLEXI_XNEST  "FLEXI_XNEST" /* <display> <xauth cookie> <xauth file> */
+#define GDM_SUP_FLEXI_XNEST  "FLEXI_XNEST" /* <display> <uid> <xauth cookie> <xauth file> */
 /* FLEXI_XNEXT: Start a new flexible Xnest server
- * Supported since: 2.2.4.2
- *   Note: supported an older version from 2.2.4.0, but since 2.2.4.2 you must
- *   supply 3 arguments or ERROR 100 will be returned.  This will start Xnest
- *   using the XAUTHORITY file supplied and as the uid same as the owner of that
- *   file.  You Must also supply the cookie as the third argument for this
+ * Supported since: 2.3.90.4
+ *   Note: supported an older version from 2.2.4.0, later 2.2.4.2, but
+ *   since 2.3.90.4 you must supply 4 arguments or ERROR 100 will be returned.
+ *   This will start Xnest  using the XAUTHORITY file supplied and as the
+ *   uid same as the owner of that file (and same as you supply).  You must
+ *   also supply the cookie as the third argument for this
  *   display, to prove that you indeed are this user.  Also this file must be
  *   readable ONLY by this user, that is have a mode of 0600.  If this all is
  *   not met, ERROR 100 is returned.
  *   Note: The cookie should be the MIT-MAGIC-COOKIE-1, the first one gdm
  *   can find in the XAUTHORITY file for this display.  If that's not what you
  *   use you should generate one first.  The cookie should be in hex form.
- * Arguments:  <display to run on> <xauth cookie for the display> <xauth file>
+ * Arguments:  <display to run on> <uid of requesting user> <xauth cookie for the display> <xauth file>
  * Answers:
  *   OK <display>
  *   ERROR <err number> <english error description>
@@ -510,6 +562,52 @@ GdmXServer *	gdm_find_x_server	(const char *id);
  *   (that is, it is an xnest inside another console display) it is listed
  *   and instead of vt, it lists the parent display in standard form.
  */
+#define GDM_SUP_UPDATE_CONFIG "UPDATE_CONFIG" /* <key> */
+/* AUTH_LOCAL: Tell the daemon to update config of some key.  Any user
+ *             can really request that values are re-read but the daemon
+ *             caches the last date of the config file so a user can't
+ *             actually change any values unless they can write the
+ *             config file.  The keys that are currently supported are:
+ *   		 security/AllowRoot (2.3.90.2)
+ *   		 security/AllowRemoteRoot (2.3.90.2)
+ *   		 security/AllowRemoteAutoLogin (2.3.90.2)
+ *   		 security/RetryDelay (2.3.90.2)
+ *   		 daemon/Greeter (2.3.90.2)
+ *   		 daemon/RemoteGreeter (2.3.90.2)
+ *   		 xdmcp/Enable (2.3.90.2)
+ *   		 xdmcp/Port (2.3.90.2)
+ *   		 xdmcp/PARAMETERS (2.3.90.2) (pseudokey, all the parameters)
+ *			xdmcp/MaxPending
+ *			xdmcp/MaxSessions
+ *			xdmcp/MaxWait
+ *			xdmcp/DisplaysPerHost
+ *			xdmcp/HonorIndirect
+ *			xdmcp/MaxPendingIndirect
+ *			xdmcp/MaxWaitIndirect
+ *			xdmcp/PingInterval (only affects new connections)
+ *   		 xdmcp/TimedLogin (2.3.90.3)
+ *   		 xdmcp/TimedLoginEnable (2.3.90.3)
+ *   		 xdmcp/TimedLoginDelay (2.3.90.3)
+ *   		 greeter/SystemMenu (2.3.90.3)
+ *   		 greeter/ConfigAvailable (2.3.90.3)
+ * Supported since: 2.3.90.2
+ * Arguments:  <key>
+ *   <key> is just the base part of the key such as "security/AllowRemoteRoot"
+ * Answers:
+ *   OK
+ *   ERROR <err number> <english error description>
+ *      0 = Not implemented
+ *      50 = Unsupported key
+ *      999 = Unknown error
+ */
+#define GDM_SUP_GREETERPIDS  "GREETERPIDS" /* None */
+/* GREETERPIDS: List all greeter pids so that one can send HUP to them
+ * for config rereading.  Of course one must be root to do that.
+ * Supported since: 2.3.90.2
+ * Arguments:  None
+ * Answers:
+ *   OK <pid>;<pid>;...
+ */
 #define GDM_SUP_CLOSE        "CLOSE" /* no arguments */
 /* CLOSE Answers: None
  * Supported since: 2.2.4.0
