@@ -54,6 +54,7 @@ static void gdm_server_spawn (GdmDisplay *d);
 static void gdm_server_usr1_handler (gint);
 static void gdm_server_alarm_handler (gint);
 static void gdm_server_child_handler (gint);
+static char * get_font_path (const char *display);
 
 /* Configuration options */
 extern gchar *GdmDisplayInit;
@@ -153,7 +154,7 @@ gdm_server_stop (GdmDisplay *disp)
 
 	    if (servpid > 0 &&
 		kill (servpid, SIGTERM) == 0)
-		    waitpid (servpid, 0, 0);
+		    ve_waitpid_no_signal (servpid, 0, 0);
 	    gdm_sigchld_block_pop ();
 
 
@@ -519,7 +520,7 @@ gdm_server_start (GdmDisplay *disp, gboolean treat_as_flexi,
 	    d->servpid = 0;
 	    if (pid > 0 &&
 		kill (pid, SIGTERM) == 0)
-		    waitpid (pid, NULL, 0);
+		    ve_waitpid_no_signal (pid, NULL, 0);
 	    gdm_sigchld_block_pop ();
     }
 
@@ -629,6 +630,70 @@ rotate_logs (const char *dname)
 	g_free (fname);
 }
 
+char **
+gdm_server_resolve_command_line (GdmDisplay *disp,
+				 gboolean resolve_handled)
+{
+	char *bin;
+	char **argv;
+	int len;
+	int i;
+
+	bin = ve_first_word (disp->command);
+	if (bin == NULL) {
+		gdm_error (_("Invalid server command '%s'"), disp->command);
+		argv = ve_split (GdmStandardXServer);
+	} else if (bin[0] != '/') {
+		GdmXServer *svr = gdm_find_x_server (bin);
+		if (svr == NULL) {
+			gdm_error (_("Server name '%s' not found, "
+				     "using standard server"), bin);
+			argv = ve_split (GdmStandardXServer);
+		} else {
+			char **svr_command =
+				ve_split (ve_sure_string (svr->command));
+			argv = ve_split (disp->command);
+			if (argv[0] == NULL || argv[1] == NULL) {
+				g_strfreev (argv);
+				argv = svr_command;
+			} else {
+				char **old_argv = argv;
+				argv = ve_vector_merge (svr_command,
+							&old_argv[1]);
+				g_strfreev (svr_command);
+				g_strfreev (old_argv);
+			} 
+
+			if (resolve_handled)
+				/* Setup the handled function */
+				disp->handled = svr->handled;
+		}
+	} else {
+		argv = ve_split (disp->command);
+	}
+
+	for (len = 0; argv != NULL && argv[len] != NULL; len++)
+		;
+
+	argv = g_renew (char *, argv, len + 4);
+	for (i = len - 1; i >= 1; i--) {
+		argv[i+1] = argv[i];
+	}
+	/* server number is the FIRST argument, before any others */
+	argv[1] = g_strdup (disp->name);
+	if (disp->authfile != NULL) {
+		argv[len+1] = g_strdup ("-auth");
+		argv[len+2] = g_strdup (disp->authfile);
+		argv[len+3] = NULL;
+	} else {
+		argv[len+1] = NULL;
+	}
+
+	g_free (bin);
+
+	return argv;
+}
+
 /**
  * gdm_server_spawn:
  * @disp: Pointer to a GdmDisplay structure
@@ -644,12 +709,9 @@ gdm_server_spawn (GdmDisplay *d)
 {
     struct sigaction ign_signal, dfl_signal;
     sigset_t mask;
-    gchar *srvcmd = NULL;
     gchar **argv = NULL;
     int logfd;
-    int len, i;
-    const char *command;
-    char *bin;
+    char *command;
     pid_t pid;
 
     if (d == NULL ||
@@ -666,49 +728,14 @@ gdm_server_spawn (GdmDisplay *d)
 	    pid_t pid = d->servpid;
 	    d->servpid = 0;
 	    if (kill (pid, SIGTERM) == 0)
-		    waitpid (pid, NULL, 0);
+		    ve_waitpid_no_signal (pid, NULL, 0);
     }
 
-
     /* Figure out the server command */
-    bin = ve_first_word (d->command);
-    if (bin == NULL) {
-	    gdm_error (_("Invalid server command '%s'"), d->command);
-	    argv = ve_split (GdmStandardXServer);
-	    command = GdmStandardXServer;
-    } else if (bin[0] != '/') {
-	    GdmXServer *svr = gdm_find_x_server (bin);
-	    if (svr == NULL) {
-		    gdm_error (_("Server name '%s' not found, "
-				 "using standard server"), bin);
-		    argv = ve_split (GdmStandardXServer);
-		    command = GdmStandardXServer;
-	    } else {
-		    char **svr_command =
-			    ve_split (ve_sure_string (svr->command));
-		    argv = ve_split (d->command);
-		    if (argv[0] == NULL || argv[1] == NULL) {
-			    g_strfreev (argv);
-			    argv = svr_command;
-		    } else {
-			    char **old_argv = argv;
-			    argv = ve_vector_merge (svr_command,
-						    &old_argv[1]);
-			    g_strfreev (svr_command);
-			    g_strfreev (old_argv);
-		    }
-		    /* this is only for information only,
-		     * so doesn't include whole command line */
-		    command = svr->command;
+    argv = gdm_server_resolve_command_line (d,
+					    TRUE /* resolve handled */);
 
-		    /* Setup the handled function */
-		    d->handled = svr->handled;
-	    }
-    } else {
-	    command = d->command;
-	    argv = ve_split (d->command);
-    }
-    g_free (bin);
+    command = g_strjoinv (" ", argv);
 
     /* Fork into two processes. Parent remains the gdm process. Child
      * becomes the X server. */
@@ -793,36 +820,36 @@ gdm_server_spawn (GdmDisplay *d)
 	sigprocmask (SIG_SETMASK, &mask, NULL);
 
 	if (d->type == TYPE_FLEXI_XNEST) {
+		char *font_path = NULL;
 		gnome_setenv ("DISPLAY", d->xnest_disp, TRUE);
 		if (d->xnest_auth_file != NULL)
 			gnome_setenv ("XAUTHORITY", d->xnest_auth_file, TRUE);
 		else
 			gnome_unsetenv ("XAUTHORITY");
+
+		/* Add -fp with the current font path, but only if not
+		 * already among the arguments */
+		if (strstr (command, "-fp") == NULL)
+			font_path = get_font_path (d->xnest_disp);
+		if (font_path != NULL) {
+			int argc = ve_vector_len (argv);
+			argv = g_renew (char *, argv, argc + 3);
+			argv[argc++] = "-fp";
+			argv[argc++] = font_path;
+			argv[argc++] = NULL;
+			command = g_strconcat (command, " -fp ",
+					       font_path, NULL);
+		}
 	}
 
-	for (len = 0; argv != NULL && argv[len] != NULL; len++)
-		;
-
-	if (len == 0) {
+	if (argv[0] == NULL) {
 		gdm_error (_("%s: Empty server command for display %s"),
 			   "gdm_server_spawn",
 			   d->name);
 		_exit (SERVER_ABORT);
 	}
 
-	argv = g_renew (char *, argv, len + 4);
-	for (i = len - 1; i >= 1; i--) {
-		argv[i+1] = argv[i];
-	}
-	/* server number is the FIRST argument, before any others */
-	argv[1] = d->name;
-	argv[len+1] = "-auth";
-	argv[len+2] = d->authfile;
-	argv[len+3] = NULL;
-
-	srvcmd = g_strjoinv (" ", argv);
-	gdm_debug ("gdm_server_spawn: '%s'", srvcmd);
-	g_free (srvcmd);
+	gdm_debug ("gdm_server_spawn: '%s'", command);
 	
 	setpgid (0, 0);
 
@@ -881,6 +908,7 @@ gdm_server_spawn (GdmDisplay *d)
 	
     case -1:
 	g_strfreev (argv);
+	g_free (command);
 	gdm_error (_("gdm_server_spawn: Can't fork Xserver process!"));
 	d->servpid = 0;
 	d->servstat = SERVER_DEAD;
@@ -889,6 +917,7 @@ gdm_server_spawn (GdmDisplay *d)
 	
     default:
 	g_strfreev (argv);
+	g_free (command);
 	gdm_debug ("gdm_server_spawn: Forked server on pid %d", (int)pid);
 	break;
     }
@@ -1005,6 +1034,7 @@ gdm_server_alloc (gint id, const gchar *command)
     gchar hostname[1024];
     GdmDisplay *d;
     
+    hostname[1023] = '\0';
     if (gethostname (hostname, 1023) == -1)
 	    strcmp (hostname, "localhost.localdomain");
 
@@ -1042,6 +1072,8 @@ gdm_server_alloc (gint id, const gchar *command)
     d->vt = -1;
 #endif
 
+    d->x_servers_order = -1;
+
     d->last_start_time = 0;
     d->retry_count = 0;
     d->disabled = FALSE;
@@ -1104,5 +1136,37 @@ gdm_server_whack_clients (GdmDisplay *disp)
 	XSetErrorHandler (old_xerror_handler);
 }
 
+static char *
+get_font_path (const char *display)
+{
+	Display *disp;
+	char **font_path;
+	int n_fonts;
+	int i;
+	GString *gs;
+
+	disp = XOpenDisplay (display);
+	if (disp == NULL)
+		return NULL;
+
+	font_path = XGetFontPath (disp, &n_fonts);
+	if (font_path == NULL) {
+		XCloseDisplay (disp);
+		return NULL;
+	}
+
+	gs = g_string_new (NULL);
+	for (i = 0; i < n_fonts; i++) {
+		if (i != 0)
+			g_string_append_c (gs, ',');
+		g_string_append (gs, font_path[i]);
+	}
+
+	XFreeFontPath (font_path);
+
+	XCloseDisplay (disp);
+
+	return g_string_free (gs, FALSE);
+}
 
 /* EOF */
