@@ -35,6 +35,10 @@ GdmDisplay *	gdm_server_alloc	(gint id,
 void		gdm_server_whack_clients (GdmDisplay *disp);
 void		gdm_server_checklog	(GdmDisplay *disp);
 
+char **		gdm_server_resolve_command_line (GdmDisplay *disp,
+						 gboolean resolve_handled);
+
+
 
 #endif /* GDM_SERVER_H */
 
