@@ -183,12 +183,9 @@ ve_is_string_in_list_case_no_locale (const GList *list, const char *string)
 }
 
 char *
-ve_find_file (const char *filename, const GList *directories)
+ve_find_file_simple (const char *filename, const GList *directories)
 {
-	char *s, *ss;
-	const char *path;
-	char **pathv;
-	int i;
+	char *s;
 	const GList *li;
 
 	if (filename == NULL)
@@ -208,6 +205,26 @@ ve_find_file (const char *filename, const GList *directories)
 		g_free (s);
 	}
 
+	return NULL;
+}
+
+char *
+ve_find_file (const char *filename, const GList *directories)
+{
+	char *s, *ss;
+	const char *path;
+	char **pathv;
+	int i;
+
+	s = ve_find_file_simple (filename, directories);
+	if (s != NULL)
+		return s;
+	/* if the filename was NULL or absolute
+	 * no further checks are made */
+	if (filename == NULL ||
+	    g_path_is_absolute (filename))
+		return NULL;
+
 	s = gnome_program_locate_file (NULL, GNOME_FILE_DOMAIN_APP_DATADIR,
 				       filename, TRUE, NULL);
 	if (s != NULL)
@@ -298,3 +315,17 @@ ve_strftime (struct tm *the_tm, const char *format)
 
 	return ve_locale_to_utf8 (str);
 }
+
+pid_t
+ve_waitpid_no_signal (pid_t pid, int *status, int options)
+{
+	pid_t ret;
+
+	for (;;) {
+		ret = waitpid (pid, status, options);
+		if (ret == 0)
+			return 0;
+		if (errno != EINTR)
+			return ret;
+	}
+}
